﻿'******************************
' アメダスデータダウンローダ、データコンバータに共通となる、初期設定周りのクラスです。
'
' [開発者]　Katsuhiro Morishita (Kumamoto-university Japan)
' [開発履歴]
'     2011/9/23   独立したモジュールとする
'     2011/9/24   何とか整備完了
'                 コンソールアプリに対応するために、メッセージボックス関係を引っ込めた。。
' *****************************
Imports System
Imports System.IO
Imports System.Text
Imports System.Text.RegularExpressions

Public Class AmedasSetting
    ' ****構造体**************************************************************************************
    Private Structure setting                                            ' 本プログラムを動かすための設定を保存する構造体
        Dim startDate As Date
        Dim stopDate As Date
        Dim targetList() As String
    End Structure
    ' ****メンバ変数**********************************************************************************
    ' 定数宣言
    Private ReadOnly AMEDAS_URL_LIST As String = "AMEDAS URL list.csv"      ' アメダスのURLリスト
    Private ReadOnly SETTING_LIST As String = "AMEDAS.ini"                  ' 初期設定リスト
    Private ReadOnly RAW_DATA_SAVE_DIR As String = "Raw HTML"               ' 生のHTML保存用ディレクトリ名前
    Private ReadOnly PROCESSED_DATA_SAVE_DIR As String = "Processed HTML"   ' 生のHTML保存用ディレクトリ名前
    Private _isOpen As Boolean = False
    Private amedasNode() As AMEDAS.Amedas                                   ' アメダスのコアデータ
    Private info As setting
    ' ****プロパティ**********************************************************************************
    ''' <summary>
    ''' ファイルが正常に開いているかを返す
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property IsOpen As Boolean
        Get
            Return Me._isOpen
        End Get
    End Property
    ''' <summary>
    ''' 格納された設定ファイルに書かれた地点数を返す
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Length As Integer
        Get
            If Me.IsOpen Then Return Me.info.targetList.Length Else Return 0
        End Get
    End Property
    ''' <summary>
    ''' 処理の開始日を返す
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property startDate As Date
        Get
            If Me.IsOpen Then Return Me.info.startDate Else Return Nothing
        End Get
    End Property
    ''' <summary>
    ''' 処理の終了日を返す
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property stopDate As Date
        Get
            If Me.IsOpen Then Return Me.info.stopDate Else Return Nothing
        End Get
    End Property
    ' ****メソッド********************************************************************************************
    ''' <summary>
    ''' HTMLファイル用に、アメダス観測点のID・地点名および時刻に基づく階層構造のフォルダを生成する
    ''' </summary>
    ''' <param name="day">日付</param>
    ''' <remarks>生成したフォルダまでのパスを返す</remarks>
    Public Function CreateDirectory4rawHTMLs(ByVal day As Date, ByVal i As Integer) As String
        Dim topdir As String = RAW_DATA_SAVE_DIR
        Dim target As String = ""
        Dim amedas As AMEDAS.Amedas

        If Me.IsOpen And Me.Length > i And i >= 0 Then                                                  ' 正常にファイルを開けたことと、指定された番号が正常かどうかを判定
            amedas = Me.amedasNode(i)                                                                   ' 指定要素のアメダスデータを代入（クラスなのでポインタのキャストに近いイメージ）
            target = topdir + "/" + amedas.ID.ToString + "_" + amedas.name + "/" + day.Year.ToString    ' ディレクトリ構造を生成
            If Not System.IO.Directory.Exists(target) Then System.IO.Directory.CreateDirectory(target) ' ディレクトリの存在を確認し、存在しない場合は生成する
        End If

        Return target
    End Function
    ''' <summary>
    ''' HTMLを加工済みのファイル用に、アメダス観測点のID・地点名および時刻に基づく階層構造のフォルダを生成する
    ''' </summary>
    ''' <param name="day">日付</param>
    ''' <remarks>生成したフォルダまでのパスを返す</remarks>
    Public Function CreateDirectory4processedHTMLs(ByVal day As Date, ByVal i As Integer) As String
        Dim topdir As String = PROCESSED_DATA_SAVE_DIR
        Dim target As String = ""
        Dim amedas As AMEDAS.Amedas

        If Me.IsOpen And Me.Length > i And i >= 0 Then                                                  ' 正常にファイルを開けたことと、指定された番号が正常かどうかを判定
            amedas = Me.amedasNode(i)                                                                   ' 指定要素のアメダスデータを代入（クラスなのでポインタのキャストに近いイメージ）
            target = topdir + "/" + amedas.ID.ToString + "_" + amedas.name + "/" + day.Year.ToString    ' ディレクトリ構造を生成
            If Not System.IO.Directory.Exists(target) Then System.IO.Directory.CreateDirectory(target) ' ディレクトリの存在を確認し、存在しない場合は生成する
        End If

        Return target
    End Function
    ''' <summary>
    ''' アメダスの気象観測データ公開URL（10分間隔の方）を生成する
    ''' </summary>
    ''' <param name="t">日付</param>
    ''' <param name="i">要素番号</param>
    ''' <returns>URL</returns>
    ''' <remarks></remarks>
    Public Function GetURL_10min(ByVal t As Date, ByVal i As Integer)
        Dim ans As String = ""

        If Me.IsOpen And Me.Length > i And i >= 0 Then ans = Me.amedasNode(i).GetURL_10min(t) ' 正常にファイルを開けたことと、指定された番号が正常かどうかを判定

        Return ans
    End Function
    ''' <summary>
    ''' HTMLファイル保存時のファイル名を返す
    ''' 何もこんなの作らなくてもと思うかもしれないが、保存したhtmlを解析するには必要なんです。
    ''' </summary>
    ''' <param name="t">日付</param>
    ''' <param name="i">要素番号</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetFileName4HTML(ByVal t As Date, ByVal i As Integer)
        Dim ans As String = ""
        Dim amedas As AMEDAS.Amedas

        If Me.IsOpen And Me.Length > i And i >= 0 Then                                                  ' 正常にファイルを開けたことと、指定された番号が正常かどうかを判定
            amedas = Me.amedasNode(i)                                                                   ' 指定要素のアメダスデータを代入
            ans = amedas.ID.ToString + "_" + amedas.name + "_" + t.Year.ToString + "_" + t.ToString("MM") + "_" + t.ToString("dd") + ".html"  ' ファイル名生成
        End If

        Return ans
    End Function
    ''' <summary>
    ''' HTMLを加工したファイルのファイル名を返す
    ''' </summary>
    ''' <param name="t">日付</param>
    ''' <param name="i">要素番号</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetFileName4Process(ByVal t As Date, ByVal i As Integer)
        Dim ans As String = ""
        Dim amedas As AMEDAS.Amedas

        If Me.IsOpen And Me.Length > i And i >= 0 Then                                                  ' 正常にファイルを開けたことと、指定された番号が正常かどうかを判定
            amedas = Me.amedasNode(i)                                                                   ' 指定要素のアメダスデータを代入
            ans = amedas.ID.ToString + "_" + amedas.name + "_" + t.Year.ToString + "_" + t.ToString("MM") + "_" + t.ToString("dd") + ".csv"  ' ファイル名生成
        End If

        Return ans
    End Function
    ''' <summary>
    ''' アメダスの観測データを公開しているURLのリストファイルを確認し、存在していた場合は呼び出し元にURLのリストを渡す
    ''' URLはyearから後ろを切り離した状態で格納される
    ''' </summary>
    ''' <param name="hs">ハッシュテーブル</param>
    ''' <returns>ファイルが存在すればture</returns>
    ''' <remarks></remarks>
    Private Function GetAMEDASurls(ByRef hs As Hashtable) As Boolean
        Dim ans As Boolean = False
        Dim line As String
        Dim field() As String

        If File.Exists(AMEDAS_URL_LIST) Then                    ' ファイルの存在を確認
            ans = True
            'AMEDAS_URL_LISTをShift-JISコードとして開く
            Using sr As New System.IO.StreamReader(AMEDAS_URL_LIST, Encoding.GetEncoding("shift_jis"))
                While sr.Peek() > -1                            ' 内容を一行ずつ読み込む
                    line = sr.ReadLine()                        ' ファイルから1行取得
                    field = line.Split(",")                     ' 文字列解析でカンマで分離させて配列に格納
                    hs(field(0)) = field(1).Split("year")(0)    ' ハッシュテーブルに地点名と"year"よりも前のurlを格納
                End While
            End Using
        Else
            Console.WriteLine("URLリストエラー")
        End If
        GetAMEDASurls = ans                                     ' 返値を格納して終了
    End Function
    ''' <summary>
    ''' 収集開始日と終了日、及び収集対象地点名を格納したファイルの存在を確認し、存在していた場合はデータを構造体に格納させる
    ''' </summary>
    ''' <param name="info">設定を保存する構造体</param>
    ''' <returns>問題なければture</returns>
    ''' <remarks></remarks>
    Private Function GetTargetList(ByRef info As setting) As Boolean
        Dim ans As Boolean = False
        Dim line As String
        Dim field() As String
        Dim year, month, day As Integer

        If File.Exists(SETTING_LIST) Then                       ' ファイルの存在を確認
            ans = True
            'AMEDAS_URL_LISTをShift-JISコードとして開く
            Using sr As New System.IO.StreamReader(SETTING_LIST, Encoding.GetEncoding("shift_jis"))
                line = sr.ReadLine                              ' 1行目取得
                field = line.Split("=")(1).Split(",")
                year = Integer.Parse(field(0))
                month = Integer.Parse(field(1))
                day = Integer.Parse(field(2))
                info.startDate = New Date(year, month, day)     ' 日付オブジェクトを生成
                line = sr.ReadLine                              ' 2行目取得
                field = line.Split("=")(1).Split(",")
                year = Integer.Parse(field(0))
                month = Integer.Parse(field(1))
                day = Integer.Parse(field(2))
                info.stopDate = New Date(year, month, day)      ' 日付オブジェクトを生成
                line = sr.ReadLine                              ' 3行目取得
                field = line.Split("=")(1).Split(",")
                info.targetList = field                         ' ダウンロード対象となる観測点名を配列で格納

                ' エラー対策
                If info.startDate > info.stopDate Then ans = False ' もしスタート日がストップ日よりも大きければ処理しない
                If info.targetList.Length = 0 Then ans = False ' 対象観測点数が0ならやはり処理しない
            End Using
        End If
        If ans = False Then Console.WriteLine("設定ファイルエラー")
        GetTargetList = ans                                     ' 返値を格納して終了
    End Function
    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        Dim urlList As Hashtable = New Hashtable                                    ' ハッシュテーブル（地域名をキーにした、URLを格納する）
        Dim targetInfo As setting = New setting                                     ' 読み込んだ設定を格納する
        Dim i, id As Integer
        Dim name, url As String

        If Me.GetAMEDASurls(urlList) And Me.GetTargetList(targetInfo) Then
            Me._isOpen = True                                                       ' 正常に設定ファイルを読み込めたことを確認する
            Me.info = targetInfo
            Me.amedasNode = New AMEDAS.Amedas(targetInfo.targetList.Length - 1) {}  ' 用意するアメダスデータ配列のサイズ決定とインスタンス生成（要素の宣言の仕方がCと違うので気持ち悪い…というか気持ち悪くなってしまった。）
            For i = 0 To Me.amedasNode.Length - 1
                name = Me.info.targetList(i)                                        ' ハッシュコードから地点名を取得
                url = urlList(name)
                id = Me.amedasNode(i).GetAMEDASid(url)
                Me.amedasNode(i) = New AMEDAS.Amedas(id, name)                      ' 改めてインスタンスを生成
            Next
        End If
    End Sub

End Class
